@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -240,11 +240,6 @@ void
 on_generic_new_seed_toggled            (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
-
-void
-on_terrain_options_activate            (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 terrain_toggle_activate                (GtkMenuShell    *menushell,
                                         gpointer         user_data);
@@ -269,10 +264,6 @@ void
 on_povray_no_clicked                   (GtkButton       *button,
                                         gpointer         user_data);
 
-void
-on_render_ok_clicked                   (GtkButton       *button,
-                                        gpointer         user_data);
-
 void
 on_terrain_render_povray_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -305,10 +296,6 @@ void
 on_generic_use_preview_toggled         (GtkToggleButton *togglebutton,
                                         gpointer         user_data);
 
-void
-on_render_apply_clicked                (GtkButton       *button,
-                                        gpointer         user_data);
-
 void
 on_save_as_ok_clicked                  (GtkButton       *button,
                                         gpointer         user_data);
@@ -768,7 +755,7 @@ on_spherical_map_activate              (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
 void
-on_apply_clicked                       (GtkButton       *button,
+on_undo_apply_clicked                       (GtkButton       *button,
                                         gpointer         user_data);
 
 void
@@ -802,3 +789,128 @@ on_terrain_view_export_file_activate   (GtkMenuItem     *menuitem,
 void
 on_terrain_view_export_gimp_activate   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_terrain_terrain_options_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_terrain_render_options_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_terrain_scene_options_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_terrain_options_ok_clicked          (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_terrain_options_apply_clicked       (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_scene_options_ok_clicked            (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_scene_options_apply_clicked         (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_terrain_options_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_terrain_options_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_use_antialiasing_toggled            (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_custom_size_toggled                 (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_use_jitter_toggled                  (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_povray_render_options_ok_clicked    (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_prune_placed_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_prune_ok_clicked                    (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_terrain_river_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+GtkWidget*
+create_terrain_preview_camera          (gchar           *widget_name, 
+                                        gchar           *string1, 
+                                        gchar           *string2,
+                                        gint             int1, 
+                                        gint             int2);
+
+void
+on_observe_sealevel_checkbutton_toggled
+                                        (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_povray_render_options_apply_clicked (GtkButton       *button,
+                                        gpointer         user_data);
+
+void
+on_terrain_all_rivers_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_terrain_redraw_rivers_activate      (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_terrain_remove_all_rivers_activate  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_method_toggled                      (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+
+void
+on_render_clouds_toggled               (GtkToggleButton *togglebutton,
+                                        gpointer         user_data);
+
+void
+on_scene_options_cancel_clicked        (GtkButton       *button,
+                                        gpointer         user_data);
+
+gboolean
+on_scene_options_window_destroy_event  (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data);
+
+gboolean
+on_scene_options_window_delete_event   (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data);
+
+void
+on_scene_options_window_destroy2       (GtkObject       *object,
+                                        gpointer         user_data);
+
+gboolean
+on_scene_options_delete_event          (GtkWidget       *widget,
+                                        GdkEvent        *event,
+                                        gpointer         user_data);
