@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,8 @@ enum TColormapType
   T_COLORMAP_LAND,
   T_COLORMAP_DESERT,
   T_COLORMAP_GRAYSCALE,
-  T_COLORMAP_HEAT
+  T_COLORMAP_HEAT,
+  T_COLORMAP_WASTELAND
 };
 
 guchar *colormap_new (TColormapType which_colormap,
