@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,6 +21,11 @@
 
 GList *theme_list_new  (void);
 GList *object_list_new (void);
+GList *cloud_list_new (void);
+GList *atmosphere_list_new (void);
+GList *star_list_new (void);
+GList *sky_list_new (void);
+GList *water_list_new (void);
 void   file_list_free  (GList *list);
 GList *file_list_copy  (GList *list);
 
