@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -164,7 +164,7 @@ t_terrain_generate_subdiv_seed_ns (gint      method,
 		    size_x + off_x - 1, size_y + off_y - 1);
     }
 
-  t_terrain_normalize (terrain, TRUE);
+  t_terrain_normalize (terrain, FALSE);
   t_terrain_set_modified (terrain, TRUE);
 
   return terrain;
