@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -46,6 +46,7 @@ static TTerrain *t_terrain_import_mat        (gchar *filename);
 static TTerrain *t_terrain_import_oct        (gchar *filename);
 static TTerrain *t_terrain_import_tga        (gchar *filename);
 static TTerrain *t_terrain_import_pgm        (gchar *filename);
+static TTerrain *t_terrain_import_ppm        (gchar *filename);
 static TTerrain *t_terrain_import_gdk_pixbuf (gchar *filename);
 static TTerrain *t_terrain_import_terragen   (gchar *filename);
 static TTerrain *t_terrain_import_grd        (gchar *filename);
@@ -79,6 +80,8 @@ t_terrain_load (gchar     *filename,
     return t_terrain_import_oct (filename);
   else if (type == FILE_PGM || type == FILE_PG8)
     return t_terrain_import_pgm (filename);
+  else if (type == FILE_PPM)
+    return t_terrain_import_ppm (filename);
   else if (type == FILE_GIF || type == FILE_ICO || type == FILE_JPG ||
            type == FILE_PNG || type == FILE_RAS || type == FILE_TIF ||
            type == FILE_XBM || type == FILE_XPM)
@@ -105,29 +108,58 @@ static void
 options_load (TTerrain   *terrain,
               xmlNodePtr  node)
 {
-  xml_unpack_float (node, "camera_x", &terrain->camera_x);
-  xml_unpack_float (node, "camera_y", &terrain->camera_y);
-  xml_unpack_float (node, "camera_z", &terrain->camera_z);
-  xml_unpack_float (node, "lookat_x", &terrain->lookat_x);
-  xml_unpack_float (node, "lookat_y", &terrain->lookat_y);
-  xml_unpack_float (node, "lookat_z", &terrain->lookat_z);
-  xml_unpack_float (node, "elevation_offset", &terrain->elevation_offset);
-  xml_unpack_boolean (node, "observe_sealevel", &terrain->observe_sealevel);
-  xml_unpack_boolean (node, "clouds", &terrain->clouds);
-  xml_unpack_boolean (node, "fog", &terrain->fog);
-  xml_unpack_boolean (node, "filled_sea", &terrain->filled_sea);
-  xml_unpack_float (node, "time_of_day", &terrain->time_of_day);
-  xml_unpack_float (node, "north_direction", &terrain->north_direction);
-  xml_unpack_float (node, "water_clarity", &terrain->water_clarity);
-  xml_unpack_int (node, "render_width", &terrain->render_width);
-  xml_unpack_float (node, "y_scale_factor", &terrain->y_scale_factor);
-  xml_unpack_float (node, "sealevel", &terrain->sealevel);
-  xml_unpack_int (node, "wireframe_resolution",
-                  &terrain->wireframe_resolution);
-  xml_unpack_string (node, "theme_file",
-                     &terrain->theme_file);
-  xml_unpack_int (node, "lighting_level", &terrain->lighting_level);
+  TTerrainOptRender *ropt = &(terrain->render_options);
+
   xml_unpack_int (node, "contour_levels", &terrain->contour_levels);
+  xml_unpack_boolean (node, "filled_sea", &terrain->do_filled_sea);
+  xml_unpack_float (node, "sealevel", &terrain->sealevel);
+  xml_unpack_int (node, "wireframe_resolution", &terrain->wireframe_resolution);
+  xml_unpack_float (node, "y_scale_factor", &terrain->y_scale_factor);
+
+  xml_unpack_boolean (node, "clouds", &ropt->do_clouds);
+  xml_unpack_boolean (node, "fog", &ropt->do_fog);
+  xml_unpack_boolean (node, "ground_fog", &ropt->do_ground_fog);
+  xml_unpack_boolean (node, "observe_sealevel", &ropt->do_observe_sealevel);
+  xml_unpack_boolean (node, "rainbow", &ropt->do_rainbow);
+
+  xml_unpack_float (node, "camera_x", &ropt->camera_x);
+  xml_unpack_float (node, "camera_y", &ropt->camera_y);
+  xml_unpack_float (node, "camera_z", &ropt->camera_z);
+  xml_unpack_float (node, "lookat_x", &ropt->lookat_x);
+  xml_unpack_float (node, "lookat_y", &ropt->lookat_y);
+  xml_unpack_float (node, "lookat_z", &ropt->lookat_z);
+  xml_unpack_float (node, "elevation_offset", &ropt->elevation_offset);
+  xml_unpack_float (node, "fog_alt", &ropt->fog_alt);
+  xml_unpack_float (node, "fog_density", &ropt->fog_density);
+  xml_unpack_float (node, "fog_offset", &ropt->fog_offset);
+  xml_unpack_float (node, "fog_turbulence", &ropt->fog_turbulence);
+  xml_unpack_float (node, "north_direction", &ropt->north_direction);
+  xml_unpack_float (node, "time_of_day", &ropt->time_of_day);
+  xml_unpack_float (node, "water_clarity", &ropt->water_clarity);
+  xml_unpack_int (node, "render_scale_x", &ropt->render_scale_x);
+  xml_unpack_int (node, "render_scale_y", &ropt->render_scale_y);
+  xml_unpack_int (node, "render_scale_z", &ropt->render_scale_z);
+
+  xml_unpack_string (node, "atmosphere_file", &ropt->atmosphere_file);
+  xml_unpack_string (node, "cloud_file", &ropt->cloud_file);
+  xml_unpack_string (node, "image_size_string", &ropt->image_size_string);
+  xml_unpack_string (node, "povray_filename", &ropt->povray_filename);
+  xml_unpack_string (node, "sky_file", &ropt->sky_file);
+  xml_unpack_string (node, "star_file", &ropt->star_file);
+  xml_unpack_string (node, "texture_file", &ropt->texture_file);
+  xml_unpack_string (node, "water_file", &ropt->water_file);
+
+  xml_unpack_boolean (node, "aa", &ropt->do_aa);
+  xml_unpack_boolean (node, "custom_size", &ropt->do_custom_size);
+  xml_unpack_boolean (node, "jitter", &ropt->do_jitter);
+  xml_unpack_int (node, "filetype", &ropt->filetype);
+  xml_unpack_int (node, "image_width", &ropt->image_width);
+  xml_unpack_int (node, "image_height", &ropt->image_height); 
+  xml_unpack_int (node, "render_quality", &ropt->render_quality);
+  xml_unpack_int (node, "aa_type", &ropt->aa_type);
+  xml_unpack_float (node, "aa_threshold", &ropt->aa_threshold);
+  xml_unpack_float (node, "aa_depth", &ropt->aa_depth);
+  xml_unpack_float (node, "jitter_amount", &ropt->jitter_amount); 
 }
 
 static gfloat*
@@ -155,6 +187,7 @@ heightfield_load_data (xmlNodePtr node, gfloat *heightfield,
 
       data = (guchar*) &heightfield[y * width];
       length = base64_decode (content, data, width * 4);
+      free (content);
       g_return_val_if_fail (length == width * 4, NULL);
 
       for (x = 0; x < width; x++)
@@ -241,6 +274,48 @@ objects_load (TTerrain   *terrain,
     }
 }
 
+static gfloat*
+riverfield_load (TTerrain *terrain, xmlNodePtr node)
+{
+  gint        width, height;
+
+  g_return_val_if_fail (sizeof (gfloat) == sizeof (guint32), NULL);
+  g_return_val_if_fail (sizeof (gfloat) == 4, NULL);
+
+  width = -1; height = -1;
+  xml_unpack_prop_int (node, "width", &width);
+  xml_unpack_prop_int (node, "height", &height);
+  g_return_val_if_fail (width == terrain->width && height == terrain->height, NULL);
+
+  if (terrain->riverfield == NULL)
+    terrain->riverfield = g_new (gfloat, width*height);
+
+  heightfield_load_data (node, terrain->riverfield, width, height);
+
+  return terrain->riverfield;
+}
+
+static void
+riversources_load (TTerrain   *terrain,
+		   xmlNodePtr  node)
+{
+  xmlNode *child;
+
+  child = node->childs;
+  while (child != NULL)
+    {
+      TTerrainRiverCoords coords;
+
+      //object.name = g_strdup (xmlGetProp (child, "name"));
+      xml_unpack_prop_float (child, "center_x", &coords.x);
+      xml_unpack_prop_float (child, "center_y", &coords.y);
+
+      g_array_append_val (terrain->riversources, coords);
+
+      child = child->next;
+    }
+}
+
 static TTerrain *
 t_terrain_load_native (gchar *filename)
 {
@@ -275,6 +350,12 @@ t_terrain_load_native (gchar *filename)
       node = xmlFindChild (doc->root, "Selection");
       if (node != NULL)
         selection_load (terrain, node);
+      node = xmlFindChild (doc->root, "Riverfield");
+      if (node != NULL)
+        riverfield_load (terrain, node);
+      node = xmlFindChild (doc->root, "Rivers");
+      if (node != NULL)
+        riversources_load (terrain, node);
 
       g_free (terrain->filename);
       terrain->filename = g_strdup (filename);
@@ -1465,9 +1546,11 @@ tga_read_image (FILE *in, struct tga_header *hdr)
         {
           short red;
           short green;
+          short blue;
 
           red = data[pos + 0];
           green = data[pos + 1];
+          blue = data[pos + 1];
 
           terrain->heightfield[i] = (red * 256.0 + green) / MAX_16_BIT;
           pos += 3;
@@ -1602,13 +1685,103 @@ t_terrain_import_pgm (gchar *filename)
           while (fscanf (in, "%s", sbuf) != EOF && points_read < npoints) 
             {
               val = strtod (sbuf, NULL);
-              terrain->heightfield[points_read++] = val / 255.0;
+              terrain->heightfield[points_read++] = val;
             }
         } 
     }
 
   fclose (in);
 
+  t_terrain_normalize (terrain, FALSE);
+  return terrain;
+}
+
+static TTerrain *
+t_terrain_import_ppm (gchar *filename)
+{
+  GtkObject *terrain_object;
+  TTerrain  *terrain = NULL;
+  FILE      *in;
+  gint32     npoints = 0;     /* # points that should be in file */
+  gint32     points_read = 0; /* # points read from hf datafile  */
+  char       sbuf[512];       /* buffer for file input           */
+  gboolean   fbin = FALSE;
+  gint       width, height;
+  gint       maxval;
+
+  in = fopen (filename, "rb");
+  if (!in)
+    return NULL;
+
+  /* *** check read *** */
+  if (fread (sbuf, sizeof (char), 2, in) != 2)
+    return NULL;
+  else
+    sbuf[2] = '\0';
+
+  if (strcmp (sbuf, "P6") == 0)     /* verify file type */
+    fbin = TRUE;
+  else
+  if (strcmp (sbuf, "P3") == 0)     /* verify file type */
+    fbin = FALSE;
+  else
+    {
+      fclose (in);
+      return NULL;
+    }
+
+  /* *** read header *** */
+  while (fscanf (in, "%s", sbuf) != EOF && sbuf[0] == '#') 
+    {
+      fscanf (in, "%[^\n]", sbuf); /* read comment beyond '#' */
+      fscanf (in, "%[\n]", sbuf);  /* read newline            */
+    }
+
+  width = atoi (sbuf);           /* store xsize of array */
+  fscanf (in, "%s", sbuf);       /* read ysize of array  */
+  height = atoi (sbuf);          /* store ysize of array */
+  fscanf (in, "%s\n", sbuf);     /* read maxval of array */
+  maxval = atoi (sbuf);          /* store maxval */
+  terrain_object = t_terrain_new (width, height);
+  terrain = T_TERRAIN (terrain_object);
+
+  /* # of points to read in */
+  npoints = width * height;
+
+  if (fbin)
+    {
+      guchar bytes[4];
+
+      while (fread (bytes, sizeof(guchar)*3, 1, in) == 1 && 
+             points_read < npoints) 
+        {
+          bytes[4] = 0x0;
+          terrain->heightfield[points_read++] = bytes[0] + 
+                                    bytes[1]*MAX_2ND_BYTE/MAX_8_BIT + 
+                                    bytes[2]*MAX_3RD_BYTE/MAX_16_BIT;
+        }
+    }
+  else
+    {
+      while (fscanf (in, "%s", sbuf) != EOF && points_read < npoints) 
+        {
+          gint v1, v2, v3; 
+
+          v1 = strtod (sbuf, NULL);
+          fscanf (in, "%s", sbuf);
+          v2 = strtod (sbuf, NULL);
+          fscanf (in, "%s", sbuf);
+          v3 = strtod (sbuf, NULL);
+
+          terrain->heightfield[points_read++] = v1/maxval*MAX_1ST_BYTE + 
+                                       v2/maxval*MAX_2ND_BYTE+MAX_8_BIT + 
+                                       v3/maxval*MAX_3RD_BYTE+MAX_16_BIT;
+        }
+    }
+
+  fclose (in);
+
+  t_terrain_normalize (terrain, FALSE);
   return terrain;
 }
 
@@ -2347,30 +2520,24 @@ t_terrain_import_e00grid (gchar *filename)
 
   FILE           *in;
   gfloat          val;
-  gint            height;
-  gint            width;
+  gint            height, width;
   gint            size;
   gint            i;
   gint            header_int;
   gfloat          header_float;
-  char            header_string[13];
+  char            header_string[13]; /* this should be long enough */
 
   in = fopen (filename, "rb");
   if (!in)
     return NULL;
 
-  /* *** check read *** */
+  /* header */
   fscanf (in, "%s %d", header_string, &width);
   fscanf (in, "%s %d", header_string, &height);
-  fscanf (in, "%s %f", header_string, &header_float);
-  fscanf (in, "%s %f", header_string, &header_float);
-  fscanf (in, "%s %d", header_string, &header_int);
-  fscanf (in, "%s %d", header_string, &header_int);
-
-  /*
-  printf ("height = %d\n", height);
-  printf ("width = %d\n", width);
-  */
+  fscanf (in, "%s %f", header_string, &header_float); /* xllcorner */
+  fscanf (in, "%s %f", header_string, &header_float); /* yllcorner */
+  fscanf (in, "%s %d", header_string, &header_int);   /* cellsize */
+  fscanf (in, "%s %d", header_string, &header_int);   /* NODATA_value */
  
   terrain_object = t_terrain_new (width, height);
   terrain = T_TERRAIN (terrain_object);
