@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,13 @@
 #include <time.h>
 #include "tterrainview.h"
 #include "contour.h"
+#include "filters.h"
 #include "gensubdiv.h"
 #include "support.h"
 #include "support2.h"
 #include "taspectframe.h"
 #include "terrainwindow.h"
+#include "trandom.h"
 
 #define SQR(x) ((x)*(x))
 
@@ -54,6 +56,10 @@ static gint t_terrain_view_button_release(GtkWidget      *widget,
                                           GdkEventButton *event);
 static gint t_terrain_view_key_press     (GtkWidget      *widget,
                                           GdkEventKey    *event);
+static gint t_terrain_view_focus_in      (GtkWidget      *widget,
+                                          GdkEventFocus  *event);
+static gint t_terrain_view_focus_out     (GtkWidget      *widget,
+                                          GdkEventFocus  *event);
 static void on_object_deleted            (GtkObject      *terrain,
                                           gint            item,
                                           GtkWidget      *user_data);
@@ -115,6 +121,8 @@ t_terrain_view_class_init (TTerrainViewClass *klass)
   widget_class->motion_notify_event  = &t_terrain_view_motion_notify;
   widget_class->button_release_event = &t_terrain_view_button_release;
   widget_class->key_press_event      = &t_terrain_view_key_press;
+  widget_class->focus_in_event       = &t_terrain_view_focus_in;
+  widget_class->focus_out_event      = &t_terrain_view_focus_out;
 }
 
 static void
@@ -275,6 +283,11 @@ t_terrain_paint_2d_plane (TTerrain *terrain,
           value = terrain->heightfield[hf_pos] * 256;
           value = MAX (MIN (value, 255), 0) * 3;
 
+	  /* rivers are blue */
+	  if (terrain->riverfield != NULL &&
+              terrain->riverfield[hf_pos] > terrain->heightfield[hf_pos])
+            value = 0;
+
           if (terrain->selection != NULL)
             {
               gint select;
@@ -337,6 +350,11 @@ t_terrain_paint_2d_contour (TTerrain     *terrain,
           value = terrain->heightfield[hf_pos] * 256;
           value = MAX (MIN (value, 255), 0) * 3;
 
+	  /* rivers are blue */
+	  if (terrain->riverfield != NULL &&
+              terrain->riverfield[hf_pos] > terrain->heightfield[hf_pos])
+            value = 0;
+
           raster[x + 0] = palette[value + 0];
           raster[x + 1] = palette[value + 1];
           raster[x + 2] = palette[value + 2];
@@ -615,7 +633,7 @@ t_terrain_paint_3d_wire (TTerrain *terrain,
           coord_x = x - half_width; coord_y = y - half_height;
           temp_x = cos_angle * coord_x + sin_angle * coord_y;
           temp_y = terrain->heightfield[pos];
-          if (terrain->filled_sea && temp_y < terrain->sealevel)
+          if (terrain->do_filled_sea && temp_y < terrain->sealevel)
             temp_y = terrain->sealevel;
           temp_y *= y_scale;
           temp_z = sin_angle * coord_x - cos_angle * coord_y;
@@ -631,7 +649,7 @@ t_terrain_paint_3d_wire (TTerrain *terrain,
               coord_x = x + division - half_width; coord_y = y - half_height;
               temp_x = cos_angle * coord_x + sin_angle * coord_y;
               temp_y = terrain->heightfield[pos + division];
-              if (terrain->filled_sea && temp_y < terrain->sealevel)
+              if (terrain->do_filled_sea && temp_y < terrain->sealevel)
                 temp_y = terrain->sealevel;
               temp_y *= y_scale;
               temp_z = sin_angle * coord_x - cos_angle * coord_y;
@@ -656,7 +674,7 @@ t_terrain_paint_3d_wire (TTerrain *terrain,
                   temp_y = terrain->heightfield[pos + yoff * terrain->width];
                 }
 
-              if (terrain->filled_sea && temp_y < terrain->sealevel)
+              if (terrain->do_filled_sea && temp_y < terrain->sealevel)
                 temp_y = terrain->sealevel;
               temp_y *= y_scale;
               temp_z = sin_angle * coord_x - cos_angle * coord_y;
@@ -743,7 +761,7 @@ t_terrain_paint_3d_light (TTerrain  *terrain,
 
       coord_x_1 = -half_width;
       coord_y_1 = terrain->heightfield[pos];
-      if (terrain->filled_sea && coord_y_1 < terrain->sealevel)
+      if (terrain->do_filled_sea && coord_y_1 < terrain->sealevel)
         coord_y_1 = terrain->sealevel;
       coord_y_1 *= y_scale;
       coord_z_1 = half_height - y;
@@ -759,7 +777,7 @@ t_terrain_paint_3d_light (TTerrain  *terrain,
           gint yoff = terrain->height - y%terrain->height - 1;
           coord_y_2 = terrain->heightfield[yoff * terrain->width];
         }
-      if (terrain->filled_sea && coord_y_2 < terrain->sealevel)
+      if (terrain->do_filled_sea && coord_y_2 < terrain->sealevel)
         coord_y_2 = terrain->sealevel;
       coord_y_2 *= y_scale;
       coord_z_2 = half_height - y - 1;
@@ -790,7 +808,7 @@ t_terrain_paint_3d_light (TTerrain  *terrain,
  
           coord_x_3 = x + 1 - half_width;
           coord_y_3 = terrain->heightfield[pos + 1];
-          if (terrain->filled_sea && coord_y_3 < terrain->sealevel)
+          if (terrain->do_filled_sea && coord_y_3 < terrain->sealevel)
             coord_y_3 = terrain->sealevel;
           coord_y_3 *= y_scale;
           coord_z_3 = half_height - y;
@@ -799,7 +817,7 @@ t_terrain_paint_3d_light (TTerrain  *terrain,
 
           coord_x_4 = x + 1 - half_width;
           coord_y_4 = terrain->heightfield[pos + terrain->width + 1];
-          if (terrain->filled_sea && coord_y_4 < terrain->sealevel)
+          if (terrain->do_filled_sea && coord_y_4 < terrain->sealevel)
             coord_y_4 = terrain->sealevel;
           coord_y_4 *= y_scale;
           coord_z_4 = half_height - y - 1;
@@ -929,7 +947,7 @@ t_terrain_paint_3d_height (TTerrain  *terrain,
 
       coord_x_1 = -half_width;
       coord_y_1 = terrain->heightfield[pos];
-      if (terrain->filled_sea && coord_y_1 < terrain->sealevel)
+      if (terrain->do_filled_sea && coord_y_1 < terrain->sealevel)
         coord_y_1 = terrain->sealevel;
       coord_y_1 *= y_scale;
       coord_z_1 = half_height - y;
@@ -944,7 +962,7 @@ t_terrain_paint_3d_height (TTerrain  *terrain,
           gint yoff = terrain->height - y % terrain->height - 1;
           coord_y_2 = terrain->heightfield[yoff * terrain->width];
         }
-      if (terrain->filled_sea && coord_y_2 < terrain->sealevel)
+      if (terrain->do_filled_sea && coord_y_2 < terrain->sealevel)
         coord_y_2 = terrain->sealevel;
       coord_y_2 *= y_scale;
       temp_x_2 = zoom * coord_x_1 / ((coord_z_1 - 1) - camera_z);
@@ -967,14 +985,14 @@ t_terrain_paint_3d_height (TTerrain  *terrain,
           coord_x_1 = x - half_width;
 
           coord_y_3 = terrain->heightfield[pos + 1];
-          if (terrain->filled_sea && coord_y_3 < terrain->sealevel)
+          if (terrain->do_filled_sea && coord_y_3 < terrain->sealevel)
             coord_y_3 = terrain->sealevel;
           coord_y_3 *= y_scale;
           temp_x_3 = zoom * (coord_x_1 + 1) / (coord_z_1 - camera_z);
           temp_y_3 = zoom * (coord_y_3 - camera_y) / (coord_z_1 - camera_z);
 
           coord_y_4 = terrain->heightfield[pos + terrain->width + 1];
-          if (terrain->filled_sea && coord_y_4 < terrain->sealevel)
+          if (terrain->do_filled_sea && coord_y_4 < terrain->sealevel)
             coord_y_4 = terrain->sealevel;
           coord_y_4 *= y_scale;
           temp_x_4 = zoom * (coord_x_1 + 1) / ((coord_z_1 - 1) - camera_z);
@@ -1287,6 +1305,15 @@ t_terrain_view_motion_notify (GtkWidget      *widget,
     t_canvas_set_selection (T_CANVAS (view), T_SELECTION_RECTANGLE,
                             view->anchor_x, view->anchor_y, x, y);
 
+  if (view->model == T_VIEW_2D_PLANE &&
+      view->tool_mode == T_TOOL_SELECT_SQUARE_SEED)
+    {
+    int size = MIN (x-view->anchor_x, y-view->anchor_y);
+    t_canvas_set_selection (T_CANVAS (view), T_SELECTION_SQUARE_SEED,
+                            view->anchor_x, view->anchor_y, 
+			    view->anchor_x+size, view->anchor_y+size);
+    }
+
   return TRUE;
 }
 
@@ -1299,6 +1326,24 @@ t_terrain_view_key_press (GtkWidget      *widget,
   return FALSE;
 }
 
+static gint
+t_terrain_view_focus_in  (GtkWidget      *widget,
+                          GdkEventFocus  *event)
+{
+  printf ("In focus in handler ...\n"); fflush (stdout);
+  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
+  return FALSE;
+}
+
+static gint
+t_terrain_view_focus_out (GtkWidget      *widget,
+                          GdkEventFocus  *event)
+{
+  printf ("In focus out handler ...\n"); fflush (stdout);
+  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
+  return FALSE;
+}
+
 
 static gint
 t_terrain_view_button_release (GtkWidget      *widget,
@@ -1315,42 +1360,43 @@ t_terrain_view_button_release (GtkWidget      *widget,
 
   if (x2 != 0 || y2 != 0)
     {
+      TTerrain  *terrain = t_terrain_view_get_terrain (T_TERRAIN_VIEW(view));
+      GtkWidget *terrain_window = lookup_toplevel (widget);
+      gint       twidth = terrain->width;
+      gint       theight = terrain->height;
+
       t_terrain_select (view->terrain, x1, y1, x2, y2, type, view->compose_op);
       t_canvas_set_selection (T_CANVAS (view), T_SELECTION_NONE, 0, 0, 0, 0);
 
       if (view->tool_mode == T_TOOL_SELECT_SQUARE_ZOOM)
         {
-        TTerrain  *terrain = t_terrain_view_get_terrain (T_TERRAIN_VIEW(view));
-        GtkWidget *terrain_window = lookup_toplevel (widget);
-	gint       twidth = terrain->width;
-	gint       theight = terrain->height;
-
-        if (t_terrain_generate_subdiv_seed_ns (0, twidth, theight, 0.5, -1, 
-			                    terrain, x1*twidth, y1*theight, 
-					    (x2-x1)*twidth, (y2-y1)*theight))
-	  {
-	    terrain_window_save_state (terrain_window, _("Zoom"));
-	  }
+        t_terrain_zoom (terrain, x1 * twidth, y1 * theight,
+                        x2 * twidth, y2 * theight);
+	terrain_window_save_state (terrain_window, _("Zoom"));
         }
       else
       if (view->tool_mode == T_TOOL_SELECT_CROP_NEW)
         {
-          TTerrain  *terrain = t_terrain_view_get_terrain (T_TERRAIN_VIEW (view));
-          GtkWidget *terrain_window;
-          GtkWidget *main_window;
-          gint       twidth = terrain->width;
-          gint       theight = terrain->height;
-
           terrain = t_terrain_crop_new (terrain, x1 * twidth, y1 * theight, 
                                         x2 * twidth, y2 * theight);
 
 	  if (terrain != NULL)
             {
+              GtkWidget *main_window;
+
               main_window = lookup_data (GTK_WIDGET (widget), "data_parent");
               terrain_window = terrain_window_new (main_window, terrain);
               terrain_window_save_state (terrain_window, _("Crop"));
             }
         }
+      else
+      if (view->tool_mode == T_TOOL_SELECT_SQUARE_SEED)
+        {
+	t_terrain_generate_subdiv_seed_ns (0, twidth, theight, 0.5, -1,
+                                           terrain, x1*twidth, y1*theight,
+                                           (x2-x1)*twidth, (y2-y1)*theight);
+	terrain_window_save_state (terrain_window, _("Seed"));
+        }
     }
 
   gtk_grab_remove (widget);
@@ -1534,6 +1580,37 @@ t_terrain_view_delete_selected_objects (TTerrainView *view)
     }
 }
 
+void 
+t_terrain_view_prune_objects (TTerrainView *view, 
+                              gfloat factor, 
+                              gint seed)
+{
+  gint      i, size;
+  TTerrain *terrain;
+  TRandom  *rnd;
+
+  terrain = view->terrain;
+
+  if (seed == -1)
+    seed = new_seed ();
+  rnd = t_random_new (seed);
+
+  terrain = view->terrain;
+  size = terrain->objects->len;
+  for (i = size - 1; i >= 0; i--)
+    {
+    TTerrainObject *obj = &g_array_index (terrain->objects, TTerrainObject, i);
+    gint pos = obj->oy * terrain->height + obj->ox;
+
+    if (terrain->selection == NULL || terrain->selection[pos] > 0.5)
+      if (t_random_rnd1(rnd) < factor)
+        on_object_deleted (GTK_OBJECT (terrain), i, GTK_WIDGET (view));
+    }
+
+  t_random_free (rnd);
+  
+}
+
 
 void
 t_terrain_view_set_terrain (TTerrainView *view,
