@@ -1,5 +1,5 @@
 /*  Physically Modeled Media Plug-In for The GIMP
- *  Copyright (c) 2000-2001 David A. Bartold
+ *  Copyright (c) 2000-2002 David A. Bartold
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 
 #include <math.h>
 #include <time.h>
+#include <stdlib.h>
 
 #include "support.h"
 #include "tworld.h"
@@ -417,6 +418,7 @@ gimp_rgb_set (GimpRGB *rgb, gdouble r, gdouble g, gdouble b)
   rgb->a = 0.0;
 }
 
+
 void
 sky_values_init (SkyValues *values,
                  gfloat     gamma)
@@ -442,6 +444,7 @@ sky_values_init (SkyValues *values,
 }
 
 
+#if 0
 /* light_shader:
  *
  * data is of type gdouble[9]:
@@ -502,35 +505,9 @@ light_shader (gpointer  data,
       }
   }
 }
-
-/* npr_shader: Non-photorealistic shader.
- *
- * data: Same parameters as light_shader ()
- */
-static void
-npr_shader (gpointer  data,
-            gdouble  *normal,
-            gdouble  *xyz,
-            gdouble  *color)
-{
-  gdouble brightness;
-  gdouble dot;
-
-  light_shader (data, normal, xyz, color);
-
-  brightness = t_vector_magnitude (color);
-
-  dot = normal[2];
-  if (fabs (dot) < 0.6 - brightness * 0.2)
-    color[0] = color[1] = color[2] = 0.0;
-
-  color[0] = RINT (color[0] * 4.0) / 4.0;
-  color[1] = RINT (color[1] * 4.0) / 4.0;
-  color[2] = RINT (color[2] * 4.0) / 4.0;
-}
+#endif
 
 
-#if 0
 /* terrain_shader: Color the terrain
  *
  * data is unused
@@ -546,8 +523,8 @@ terrain_shader (gpointer  data,
   gdouble *solid;
   gdouble  light[3] = { -0.57735, 0.57735, -0.57735 };
   gdouble  snow_color[3] = { 0.95, 0.95, 0.95 };
-  gdouble  grass_color[3] = { 0.15, 0.4, 0.01 };
-  gdouble  dirt_color[3] = { 0.27, 0.25, 0.15 };
+  gdouble  grass_color[3] = { 0.09, 0.3, 0.03 };
+  gdouble  dirt_color[3] = { 0.27, 0.24, 0.1 };
   gdouble  dot;
   TVector  mirror;
   TVector  camera;
@@ -564,17 +541,17 @@ terrain_shader (gpointer  data,
   else if (normal[1] <= 0.55)
     {
       /* Dirt */
-      ambient = 0.5;
-      diffuse = 0.5;
+      ambient = 0.3;
+      diffuse = 0.7;
       specular = 0.0;
       solid = dirt_color;
     }
   else
     {
       /* Grass */
-      ambient = 0.4;
-      diffuse = 0.6;
-      specular = 0.1;
+      ambient = 0.3;
+      diffuse = 0.5;
+      specular = 0.2;
       solid = grass_color;
     }
 
@@ -613,85 +590,212 @@ terrain_shader (gpointer  data,
       color[2] += dot;
     }
 }
-#endif
 
 
-#if 0
-/* torus: Create a torus around the y axis
- * data is of type gdouble[2]:
- * [0] = ring radius
- * [1] = thickness
- */
-static void
-torus (gpointer  data,
-       gdouble  *uv,
-       gdouble  *xyz)
+typedef struct _NativeTerrain NativeTerrain;
+
+struct _NativeTerrain
 {
-  gdouble *params = (gdouble *) data;
-  gdouble  x, y;
+  NativePaint  paint;
+  TTerrain    *terrain;
+};
 
-  x = params[0] + cos (uv[1] * 2.0 * G_PI) * params[1];
-  y = -sin (uv[1] * 2.0 * G_PI) * params[1];
-
-  xyz[0] = cos (uv[0] * 2.0 * G_PI) * x;
-  xyz[1] = y;
-  xyz[2] = sin (uv[0] * 2.0 * G_PI) * x;
-}
-#endif
-
-
-/* crazy_torus: Create a wacky torus around the y axis
- * data is of type gdouble[5]:
- * [0] = ring radius
- * [1] = thickness min
- * [2] = thickness max
- * [3] = thickness frequency
- * [4] = star frequency
- */
-static void
-crazy_torus (gpointer  data,
-             gdouble  *uv,
-             gdouble  *xyz)
+/* get_random: Return a random number given two doubles used as seeds */
+static gdouble
+get_random (gdouble x, gdouble y)
 {
-  gdouble *params = (gdouble *) data;
-  gdouble  x, y;
+  guint32 *a = (guint32*) &x;
+  guint32 *b = (guint32*) &y;
 
-  gdouble  thickness;
+  srand (((a[0] + a[1]) ^ 0x2f93a473) + ((b[0] + b[1]) ^ 0xfca814ed));
 
-  thickness = (sin (uv[0] * 2.0 * G_PI * params[3]) + 1.0) * 0.5 *
-              (params[2] - params[1]) + params[1];
-
-  thickness *= (sin (uv[1] * 2.0 * G_PI * params[4]) + 1.0) * 0.4 + 0.2;
-
-  x = params[0] + cos (uv[1] * 2.0 * G_PI) * thickness;
-  y = -sin (uv[1] * 2.0 * G_PI) * thickness;
-
-  xyz[0] = cos (uv[0] * 2.0 * G_PI) * x;
-  xyz[1] = y;
-  xyz[2] = sin (uv[0] * 2.0 * G_PI) * x;
+  return (rand () + 0.0) / RAND_MAX;
 }
 
 
-#if 0
-/* terrain_object: Draw a terrain on the xz axis centered around the origin
- *
- * data is of type TTerrain
- */
+static gdouble
+midpoint (gdouble  location_x,
+          gdouble  location_y,
+          gdouble  value1,
+          gdouble  value2)
+{
+  gdouble rand;
+  gdouble x, x2;
+  gdouble mean, deviation;
+
+  x = value1 + value2;
+  x2 = SQR (value1) + SQR (value2);
+
+  mean = x / 2.0;
+  deviation = sqrt (x2 - x * x / 2.0);
+
+  rand = get_random (location_x, location_y) - 0.5;
+
+  return mean + rand * deviation;
+}
+
+
+static gboolean
+split_triangle (gdouble xy_triangle[3][2])
+{
+  return fabs (xy_triangle[0][0] - xy_triangle[1][0]) >= 1.0 ||
+         fabs (xy_triangle[1][0] - xy_triangle[2][0]) >= 1.0 ||
+         fabs (xy_triangle[2][0] - xy_triangle[0][0]) >= 1.0 ||
+         fabs (xy_triangle[0][1] - xy_triangle[1][1]) >= 1.0 ||
+         fabs (xy_triangle[1][1] - xy_triangle[2][1]) >= 1.0 ||
+         fabs (xy_triangle[2][1] - xy_triangle[0][1]) >= 1.0;
+}
+
 
 static void
-terrain_object (gpointer  data,
-                gdouble  *uv,
-                gdouble  *xyz)
+terrain_square_render_b (NativeTerrain *terrain,
+                         Region        *region,
+                         gint           depth,
+                         gdouble        min_x,
+                         gdouble        max_x,
+                         gdouble        min_y,
+                         gdouble        max_y,
+                         gdouble        a,
+                         gdouble        b,
+                         gdouble        c,
+                         gdouble        d)
 {
-  TTerrain *terrain = (TTerrain*) data;
+  NativePaint *state = &terrain->paint;
+  gboolean     split;
+  gdouble      points[4][3];
+  gdouble      temp_xy[4][2];
+  gint         i;
 
-  /* FIXME: Render non-square terrains correctly */
+  if (depth == 0)
+    return;
 
-  xyz[0] = uv[0] - 0.5;
-  xyz[1] = t_terrain_get_height (terrain, uv[0], uv[1]) * terrain->y_scale_factor;
-  xyz[2] = -(uv[1] - 0.5);
+  points[0][0] = min_x - 0.5;
+  points[0][1] = a;
+  points[0][2] = 0.5 - min_y;
+  points[1][0] = max_x - 0.5;
+  points[1][1] = b;
+  points[1][2] = 0.5 - min_y;
+  points[2][0] = min_x - 0.5;
+  points[2][1] = c;
+  points[2][2] = 0.5 - max_y;
+  points[3][0] = max_x - 0.5;
+  points[3][1] = d;
+  points[3][2] = 0.5 - max_y;
+
+  for (i = 0; i < 4; i++)
+    {
+      gdouble temp_xyz[3];
+
+      state->object_transform (state->object_transform_data,
+                               points[i], temp_xyz);
+      state->projection (state->projection_data, temp_xyz, temp_xy[i]);
+    }
+
+  /* If upper triangle doesn't fit a pixel, split square */
+  split = split_triangle (&temp_xy[0]);
+
+  /* If lower triangle doesn't fit a pixel, split square */
+  if (split == FALSE)
+    split = split_triangle (&temp_xy[1]);
+
+  if (split)
+    {
+      /* Split square */
+
+      gdouble h, i, j, k, l;
+      gdouble mid_x, mid_y;
+      gdouble rand;
+      gdouble x, x2;
+      gdouble mean, deviation;
+
+      mid_x = (min_x + max_x) * 0.5;
+      mid_y = (min_y + max_y) * 0.5;
+
+      x = a + b + c + d;
+      x2 = a * a + b * b + c * c + d * d;
+      mean = x / 4.0;
+      deviation = sqrt ((x2 - x * x / 4.0) / 3.0);
+      rand = get_random (mid_x, mid_y) - 0.5;
+
+      /*
+       * ahb
+       * ijk
+       * cld
+       */
+
+      h = midpoint (mid_x, min_y, a, b);
+      i = midpoint (min_x, mid_y, a, c);
+      j = mean + rand * deviation;
+      k = midpoint (max_x, mid_y, b, d);
+      l = midpoint (mid_x, max_y, c, d);
+
+      terrain_square_render_b (terrain, region, depth - 1,
+                               min_x, mid_x, min_y, mid_y,
+                               a, h, i, j);
+      terrain_square_render_b (terrain, region, depth - 1,
+                               mid_x, max_x, min_y, mid_y,
+                               h, b, j, k);
+      terrain_square_render_b (terrain, region, depth - 1,
+                               min_x, mid_x, mid_y, max_y,
+                               i, j, c, l);
+      terrain_square_render_b (terrain, region, depth - 1,
+                               mid_x, max_x, mid_y, max_y,
+                               j, k, l, d);
+    }
+  else
+    {
+      /* Otherwise paint triangles */
+
+      gdouble upper_xyz[3][3];
+      gdouble lower_xyz[3][3];
+
+      t_vector_copy (points[0], upper_xyz[0]);
+      t_vector_copy (points[1], upper_xyz[1]);
+      t_vector_copy (points[2], upper_xyz[2]);
+
+      t_vector_copy (points[1], lower_xyz[0]);
+      t_vector_copy (points[3], lower_xyz[1]);
+      t_vector_copy (points[2], lower_xyz[2]);
+
+      paint_triangle (state, region, upper_xyz);
+      paint_triangle (state, region, lower_xyz);
+    }
+}
+
+
+static void
+terrain_square_render (NativeTerrain *state,
+                       Region        *region,
+                       gint           x,
+                       gint           y)
+{
+  gfloat   *pos;
+  TTerrain *terrain = state->terrain;
+
+  pos = &terrain->heightfield[y * terrain->width + x];
+
+  terrain_square_render_b (state, region, 14,
+                           ((gdouble) x) / terrain->width,
+                           ((gdouble) (x + 1)) / terrain->width,
+                           ((gdouble) y) / terrain->height,
+                           ((gdouble) (y + 1)) / terrain->height,
+                           pos[0] * terrain->y_scale_factor,
+                           pos[1] * terrain->y_scale_factor,
+                           pos[terrain->width] * terrain->y_scale_factor,
+                           pos[terrain->width + 1] * terrain->y_scale_factor);
+}
+
+
+static void
+terrain_render (NativeTerrain *terrain,
+                Region        *region)
+{
+  gint     x, y;
+
+  for (y = 0; y < terrain->terrain->height - 1; y++)
+    for (x = 0; x < terrain->terrain->width - 1; x++)
+      terrain_square_render (terrain, region, x, y);
 }
-#endif
 
 
 void
@@ -732,23 +836,17 @@ sky_render (TTerrain     *terrain,
   /* Draw the terrain */
 
   {
-    PaintState state;
-    TMatrix    matrix_1, matrix_2;
-    gdouble    projection_data[4];
-    gdouble    shader_data_1[9] = { 0.8, 0.15, 0.025, -0.57735, 0.57735, -0.57735, 0.2, 0.8, 1.0 };
-    gdouble    shader_data_2[9] = { 0.025, 0.8, 0.15, -0.57735, 0.57735, -0.57735, 0.05, 0.95, 0.5 };
-    gdouble    crazy_torus_data[5] = { 0.8, 0.1, 0.3, 11.0, 5.0 };
+    NativeTerrain  terrain_state;
+    NativePaint   *paint = &terrain_state.paint;
+    TMatrix        matrix_1;
+    gdouble        projection_data[4];
 
     {
       TMatrix rotate, translate;
 
-      t_matrix_translate (translate, 0.0, 0.0, 10.0);
-/*      t_matrix_translate (translate, 0, -0.2, 1.2); */
-      t_matrix_rotate (rotate, 20.0, 1);
+      t_matrix_translate (translate, 0.0, -0.25, 4.0);
+      t_matrix_rotate (rotate, 30.0, 1);
       t_matrix_combine (rotate, translate, matrix_1);
-
-      t_matrix_rotate (rotate, -70.0, 1);
-      t_matrix_combine (rotate, translate, matrix_2);
     }
 
     projection_data[0] = pixbuf->width;
@@ -756,40 +854,28 @@ sky_render (TTerrain     *terrain,
     projection_data[2] = MIN (pixbuf->width, pixbuf->height) / 1.1;
     projection_data[3] = 5.0;
 
-    state.gamma                 = values->gamma;
-    /* state.object                = &terrain_object; */
-    state.object                = &crazy_torus;
-    state.projection            = &perspective_projection;
-    state.object_transform      = &affine_transform;
-    state.shader_transform      = &affine_transform;
-    /* state.shader                = &terrain_shader; */
-    state.shader                = &npr_shader;
-    /* state.object_data           = terrain; */
-    state.object_data           = crazy_torus_data;
-    state.projection_data       = &projection_data;
-    state.object_transform_data = matrix_1;
-    state.shader_transform_data = matrix_1;
-    /* state.shader_data           = NULL; */
-    state.shader_data           = shader_data_1;
+    terrain_state.terrain        = terrain;
+    paint->gamma                 = values->gamma;
+    paint->projection            = &perspective_projection;
+    paint->object_transform      = &affine_transform;
+    paint->shader_transform      = &affine_transform;
+    paint->shader                = &terrain_shader;
+    paint->projection_data       = &projection_data;
+    paint->object_transform_data = matrix_1;
+    paint->shader_transform_data = matrix_1;
+    paint->shader_data           = NULL;
 
     region.data = pixbuf->data;
-    region.y = 0;
-    region.h = pixbuf->height;
+    region.y    = 0;
+    region.h    = pixbuf->height;
 
     /* Initialize the Z buffer */
     region.zbuffer = g_new (gfloat, region.w * region.h);
     for (index = 0; index < region.w * region.h; index++)
       region.zbuffer[index] = 10e20;
 
-    /* Paint red NPR torus */
-    paint_object (&region, &state);
-
-    /* Paint green torus */
-    state.shader                = &light_shader;
-    state.object_transform_data = matrix_2;
-    state.shader_transform_data = matrix_2;
-    state.shader_data           = shader_data_2;
-    paint_object (&region, &state);
+    /* Paint terrain */
+    terrain_render (&terrain_state, &region);
 
     g_free (region.zbuffer);
   }
