@@ -1,4 +1,4 @@
-/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
+/* Terraform - (C) 1997-2002 Robert Gasch (r.gasch@chello.nl)
  *  - http://terraform.sourceforge.net
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -19,8 +19,11 @@
 
 #include <locale.h> 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
+#include <sys/stat.h>
 #include "tterrainview.h"
 #include "writer.h"
 #include "reader.h"        /* defines BT file header */
@@ -29,13 +32,19 @@
 #include "base64.h"
 #include "xmlsupport.h"
 #include "clocale.h"
+#include "support2.h"
 
 
 static gint     t_terrain_save_native     (TTerrain  *terrain);
+static gint     t_terrain_export_tga_field(TTerrain  *terrain,
+                                           gchar     *filename, 
+                                           gfloat    *field);
 static gint     t_terrain_export_tga      (TTerrain  *terrain,
                                            gchar     *filename);
 static gint     t_terrain_export_pov      (TTerrain  *terrain,
                                            gchar     *filename);
+static gint     t_terrain_export_ini      (TTerrain  *terrain,
+                                           gchar     *filename);
 static gint     t_terrain_export_bmp      (TTerrain  *terrain,
                                            gchar     *filename);
 static gint     t_terrain_export_bmp_bw   (TTerrain  *terrain,
@@ -44,6 +53,8 @@ static gint     t_terrain_export_pgm      (TTerrain  *terrain,
                                            gchar     *filename);
 static gint     t_terrain_export_pg8      (TTerrain  *terrain,
                                            gchar     *filename);
+static gint     t_terrain_export_ppm      (TTerrain  *terrain,
+                                           gchar     *filename);
 static gint     t_terrain_export_mat      (TTerrain  *terrain,
                                            gchar     *filename);
 static gint     t_terrain_export_oct      (TTerrain  *terrain,
@@ -83,6 +94,8 @@ t_terrain_save (TTerrain  *terrain,
     return t_terrain_export_tga (terrain, filename);
   else if (type == FILE_POV)
     return t_terrain_export_pov (terrain, filename);
+  else if (type == FILE_INI)
+    return t_terrain_export_ini (terrain, filename);
   else if (type == FILE_BMP)
     return t_terrain_export_bmp (terrain, filename);
   else if (type == FILE_BMP_BW)
@@ -91,6 +104,8 @@ t_terrain_save (TTerrain  *terrain,
     return t_terrain_export_pgm (terrain, filename);
   else if (type == FILE_PG8)
     return t_terrain_export_pg8 (terrain, filename);
+  else if (type == FILE_PPM)
+    return t_terrain_export_ppm (terrain, filename);
   else if (type == FILE_MAT)
     return t_terrain_export_mat (terrain, filename);
   else if (type == FILE_OCT)
@@ -133,9 +148,9 @@ t_terrain_get_povray_size (TTerrain *terrain,
 
   max_size = MAX (terrain->width, terrain->height);
 
-  *x = terrain->render_scale_x * terrain->width / max_size;
-  *y = terrain->render_scale_y;
-  *z = terrain->render_scale_z * terrain->height / max_size;
+  *x = terrain->render_options.render_scale_x * terrain->width / max_size;
+  *z = terrain->render_options.render_scale_z * terrain->height / max_size;
+  *y = (*x + *z)/2 * terrain->y_scale_factor;
 }
 
 static void 
@@ -187,6 +202,19 @@ heightfield_save (TTerrain   *terrain,
   t_terrain_set_modified (terrain, FALSE);
 }
 
+static void
+riverfield_save (TTerrain   *terrain,
+                 xmlNodePtr  node)
+{
+  g_return_if_fail (sizeof (gfloat) == sizeof (guint32));
+  g_return_if_fail (sizeof (gfloat) == 4);
+
+  heightfield_save_data (node, terrain->riverfield, 
+                         terrain->width, terrain->height);
+
+  t_terrain_set_modified (terrain, FALSE);
+}
+
 static void
 selection_save (TTerrain   *terrain,
                 xmlNodePtr  node)
@@ -204,45 +232,76 @@ static void
 options_save (TTerrain   *terrain,
               xmlNodePtr  node)
 {
-  xml_pack_float (node, "camera_x", terrain->camera_x);
-  xml_pack_float (node, "camera_y", terrain->camera_y);
-  xml_pack_float (node, "camera_z", terrain->camera_z);
-  xml_pack_float (node, "lookat_x", terrain->lookat_x);
-  xml_pack_float (node, "lookat_y", terrain->lookat_y);
-  xml_pack_float (node, "lookat_z", terrain->lookat_z);
-  xml_pack_float (node, "elevation_offset", terrain->elevation_offset);
-  xml_pack_boolean (node, "observe_sealevel", terrain->observe_sealevel);
-  xml_pack_boolean (node, "clouds", terrain->clouds);
-  xml_pack_boolean (node, "fog", terrain->fog);
-  xml_pack_boolean (node, "filled_sea", terrain->filled_sea);
-  xml_pack_float (node, "time_of_day", terrain->time_of_day);
-  xml_pack_float (node, "north_direction", terrain->north_direction);
-  xml_pack_float (node, "water_clarity", terrain->water_clarity);
-  xml_pack_int (node, "render_width", terrain->render_width);
-  xml_pack_float (node, "y_scale_factor", terrain->y_scale_factor);
-  xml_pack_float (node, "sealevel", terrain->sealevel);
-  xml_pack_int (node, "wireframe_resolution",
-                terrain->wireframe_resolution);
-  xml_pack_string (node, "theme_file",
-                   terrain->theme_file);
-  xml_pack_int (node, "lighting_level", terrain->lighting_level);
+  TTerrainOptRender * const ropt = &(terrain->render_options);
+
   xml_pack_int (node, "contour_levels", terrain->contour_levels);
+  xml_pack_boolean (node, "filled_sea", terrain->do_filled_sea);
+  xml_pack_float (node, "sealevel", terrain->sealevel);
+  xml_pack_int (node, "wireframe_resolution", terrain->wireframe_resolution);
+  xml_pack_float (node, "y_scale_factor", terrain->y_scale_factor);
+
+  xml_pack_boolean (node, "clouds", ropt->do_clouds);
+  xml_pack_boolean (node, "fog", ropt->do_fog);
+  xml_pack_boolean (node, "ground_fog", ropt->do_ground_fog);
+  xml_pack_boolean (node, "observe_sealevel", ropt->do_observe_sealevel);
+  xml_pack_boolean (node, "rainbow", ropt->do_rainbow);
+
+  xml_pack_float (node, "camera_x", ropt->camera_x);
+  xml_pack_float (node, "camera_y", ropt->camera_y);
+  xml_pack_float (node, "camera_z", ropt->camera_z);
+  xml_pack_float (node, "lookat_x", ropt->lookat_x);
+  xml_pack_float (node, "lookat_y", ropt->lookat_y);
+  xml_pack_float (node, "lookat_z", ropt->lookat_z);
+  xml_pack_float (node, "elevation_offset", ropt->elevation_offset);
+  xml_pack_float (node, "fog_alt", ropt->fog_alt);
+  xml_pack_float (node, "fog_density", ropt->fog_density);
+  xml_pack_float (node, "fog_offset", ropt->fog_offset);
+  xml_pack_float (node, "fog_turbulence", ropt->fog_turbulence);
+  xml_pack_float (node, "north_direction", ropt->north_direction);
+  xml_pack_float (node, "time_of_day", ropt->time_of_day);
+  xml_pack_float (node, "water_clarity", ropt->water_clarity);
+  xml_pack_int (node, "render_scale_x", ropt->render_scale_x);
+  xml_pack_int (node, "render_scale_y", ropt->render_scale_y);
+  xml_pack_int (node, "render_scale_z", ropt->render_scale_z);
+
+  xml_pack_string (node, "atmosphere_file", ropt->atmosphere_file); 
+  xml_pack_string (node, "cloud_file", ropt->cloud_file); 
+  xml_pack_string (node, "image_size_string", ropt->image_size_string); 
+  xml_pack_string (node, "povray_filename", ropt->povray_filename); 
+  xml_pack_string (node, "sky_file", ropt->sky_file); 
+  xml_pack_string (node, "star_file", ropt->star_file); 
+  xml_pack_string (node, "texture_file", ropt->texture_file); 
+  xml_pack_string (node, "water_file", ropt->water_file); 
+
+  xml_pack_boolean (node, "aa", ropt->do_aa);
+  xml_pack_boolean (node, "custom_size", ropt->do_custom_size);
+  xml_pack_boolean (node, "jitter", ropt->do_jitter);
+  xml_pack_string (node, "povray_filename", ropt->povray_filename);
+  xml_pack_int (node, "filetype", ropt->filetype);
+  xml_pack_int (node, "image_width", ropt->image_width);
+  xml_pack_int (node, "image_height", ropt->image_height);
+  xml_pack_int (node, "render_quality", ropt->render_quality);
+  xml_pack_float (node, "aa_threshold", ropt->aa_threshold);
+  xml_pack_int (node, "aa_type", ropt->aa_type);
+  xml_pack_int (node, "aa_threshold", ropt->aa_type);
+  xml_pack_float (node, "aa_depth", ropt->aa_depth);
+  xml_pack_float (node, "jitter_amount", ropt->jitter_amount);
 }
 
 static void
 objects_save (TTerrain   *terrain,
               xmlNodePtr  node)
 {
-  GArray *array;
+  GArray *object_array;
   gint    i;
 
-  array = terrain->objects;
-  for (i = 0; i < array->len; i++)
+  object_array = terrain->objects;
+  for (i = 0; i < object_array->len; i++)
     {
       TTerrainObject *object;
       xmlNodePtr      xml_object;
 
-      object = &g_array_index (array, TTerrainObject, i);
+      object = &g_array_index (object_array, TTerrainObject, i);
       xml_object = xmlNewChild (node, NULL, "object", NULL);
 
       xmlSetProp (xml_object, "name", object->name);
@@ -257,6 +316,28 @@ objects_save (TTerrain   *terrain,
     }
 }
 
+static void
+riversources_save (TTerrain   *terrain,
+		   xmlNodePtr  node)
+{
+  GArray *source_array;
+  gint    i;
+
+  source_array = terrain->riversources;
+  for (i = 0; i < source_array->len; i++)
+    {
+      TTerrainRiverCoords *coords;
+      xmlNodePtr      xml_object;
+
+      coords = &g_array_index (source_array, TTerrainRiverCoords, i);
+      xml_object = xmlNewChild (node, NULL, "riversource", NULL);
+
+      xmlSetProp (xml_object, "name", "river");
+      xml_pack_prop_float (xml_object, "center_x", coords->x);
+      xml_pack_prop_float (xml_object, "center_y", coords->y);
+    }
+}
+
 static gint
 t_terrain_save_native (TTerrain *terrain)
 {
@@ -269,8 +350,12 @@ t_terrain_save_native (TTerrain *terrain)
   options_save (terrain, node);
   node = xmlNewChild (doc->root, NULL, "Heightfield", NULL);
   heightfield_save (terrain, node);
-  node = xmlNewChild (doc->root, NULL, "Objects", NULL);
-  objects_save (terrain, node);
+
+  if (terrain->objects->len)
+    {
+    node = xmlNewChild (doc->root, NULL, "Objects", NULL);
+    objects_save (terrain, node);
+    }
 
   if (terrain->selection)
     {
@@ -278,6 +363,18 @@ t_terrain_save_native (TTerrain *terrain)
     selection_save (terrain, node);
     }
 
+  if (terrain->riverfield)
+    {
+    node = xmlNewChild (doc->root, NULL, "Riverfield", NULL);
+    riverfield_save (terrain, node);
+    }
+
+  if(terrain->riversources->len > 0)
+    {
+    node = xmlNewChild (doc->root, NULL, "Rivers", NULL);
+    riversources_save (terrain, node);
+    }
+
   xmlSaveFile (terrain->filename, doc);
   xmlFreeDoc (doc);
 
@@ -285,13 +382,14 @@ t_terrain_save_native (TTerrain *terrain)
 }
 
 static gint
-t_terrain_export_tga (TTerrain *terrain,
-                      gchar    *filename)
+t_terrain_export_tga_field (TTerrain *terrain,
+                            gchar    *filename, 
+                            gfloat   *field)
 {
   FILE  *out;
   gchar  buf[3];
   gint   i, size;
-  gint   status;
+  gint   status = 0;
 
   out = fopen (filename, "wb");
   if (!out)
@@ -323,7 +421,7 @@ t_terrain_export_tga (TTerrain *terrain,
       gint value;
 
       value =
-        (gint) MIN (MAX (MAX_16_BIT * terrain->heightfield[i], 0.0), MAX_16_BIT);
+        (gint) MIN (MAX (MAX_16_BIT * field[i], 0.0), MAX_16_BIT);
 
       buf[1] = value;      /* Green */
       buf[2] = value >> 8; /* Red   */
@@ -337,6 +435,14 @@ t_terrain_export_tga (TTerrain *terrain,
   return status;
 }
 
+static gint
+t_terrain_export_tga (TTerrain *terrain,
+                      gchar    *filename) 
+{
+  return t_terrain_export_tga_field (terrain, filename, terrain->heightfield);
+
+}
+
 static gint
 t_terrain_export_pov (TTerrain *terrain,
                       gchar    *filename)
@@ -344,11 +450,13 @@ t_terrain_export_pov (TTerrain *terrain,
   FILE   *out;
   time_t  now;
   gchar  *tga;
+  gchar  *tmp;
   gfloat  size_x, size_y, size_z;
-  gint    status;
+  gint    status = 0;
   gint    i;
-  GArray *array;
+  GArray *object_array;
   GTree  *includes;
+  TTerrainOptRender * const ropt = &(terrain->render_options);
 
   /* save the current locale so that the povray files are written 
    * with the correct format
@@ -362,74 +470,75 @@ t_terrain_export_pov (TTerrain *terrain,
   now = time (NULL);
 
   fprintf (out, "// File created by Terraform on %s\n", ctime (&now));
-  tga = filename_new_extension (filename, "tga");
+  tmp = filename_get_base_without_extension_and_dot (filename);
+  tga = g_strdup_printf ("%s_hf.tga", tmp);
+  g_free (tmp);
+
   fprintf (out, "#declare TF_HEIGHT_FIELD = height_field { tga \"%s\" smooth }\n",
            tga);
-  g_free (tga);
   //fprintf (out, "#declare TF_TILE_TERRAIN = %s;\n", terrain->do_tile ? "true" : "false");
   fprintf (out, "#declare TF_TILE_TERRAIN = %s;\n", "false");
   fprintf_C (out, "#declare TF_WATER_LEVEL = %f;\n", terrain->sealevel);
-  fprintf (out, "#declare TF_HAVE_WATER = %s;\n", terrain->filled_sea ? "true" : "false");
-  fprintf_C (out, "#declare TF_WATER_CLARITY = %f;\n", terrain->water_clarity);
-  fprintf_C (out, "#declare TF_TIME_OF_DAY = %f;\n", terrain->time_of_day);
-  fprintf (out, "#declare TF_HAVE_CLOUDS = %s;\n", terrain->clouds ? "true" : "false");
-  fprintf (out, "#declare TF_HAVE_FOG = %s;\n", terrain->fog ? "true" : "false");
-  fprintf (out, "#declare TF_FOG_OFFSET = %f;\n", terrain->fog_offset);
-  fprintf (out, "#declare TF_FOG_ALT = %f;\n", terrain->fog_alt);
-  fprintf (out, "#declare TF_FOG_DISTANCE = %f;\n", terrain->fog_distance);
-  fprintf_C (out, "#declare TF_NORTH_DIR = %f;\n", terrain->north_direction);
+  fprintf (out, "#declare TF_HAVE_WATER = %s;\n", terrain->do_filled_sea ? "true" : "false");
+  fprintf_C (out, "#declare TF_WATER_CLARITY = %f;\n", ropt->water_clarity);
+  fprintf_C (out, "#declare TF_TIME_OF_DAY = %f;\n", ropt->time_of_day);
+  fprintf (out, "#declare TF_HAVE_CLOUDS = %s;\n", ropt->do_clouds ? "true" : "false");
+  fprintf (out, "#declare TF_CLOUDS_TEXTURE = \"%s\"\n", ropt->cloud_file);
+  fprintf (out, "#declare TF_STARS_TEXTURE = \"%s\"\n", ropt->star_file);
+  fprintf (out, "#declare TF_HAVE_FOG = %s;\n", ropt->do_fog ? "true" : "false");
+  fprintf (out, "#declare TF_HAVE_GROUND_FOG = %s;\n", ropt->do_ground_fog ? "true" : "false");
+  fprintf (out, "#declare TF_HAVE_RAINBOW = %s;\n", ropt->do_rainbow ? "true" : "false");
+  fprintf (out, "#declare TF_FOG_OFFSET = %f;\n", ropt->fog_offset);
+  fprintf (out, "#declare TF_FOG_ALT = %f;\n", ropt->fog_alt);
+  fprintf (out, "#declare TF_FOG_TURBULENCE = %f;\n", ropt->fog_turbulence);
+  fprintf (out, "#declare TF_FOG_DENSITY = %f;\n", ropt->fog_density);
+  fprintf_C (out, "#declare TF_NORTH_DIR = %f;\n", ropt->north_direction);
 
   fprintf_C (out, "#declare TF_Y_SCALE_FACTOR = %f;\n", terrain->y_scale_factor);
 
-  if (terrain->lighting_level == 0)
-    fprintf (out, "#declare TF_REAL_LIGHTING = false;\n");
-  else
-    fprintf (out, "#declare TF_REAL_LIGHTING = true;\n");
-
-  if (terrain->lighting_level > 1)
-    fprintf (out, "#declare TF_HAVE_SUNBEAMS = true;\n");
-  else
-    fprintf (out, "#declare TF_HAVE_SUNBEAMS = false;\n");
-
-  if (terrain->lighting_level > 2)
-    fprintf (out, "#declare TF_FAST_SUNBEAMS = false;\n");
-  else
-    fprintf (out, "#declare TF_FAST_SUNBEAMS = true;\n");
-
   /* write any objects which are to be placed to the POV file */
   fprintf (out, "\n\n");
 
   t_terrain_get_povray_size (terrain, &size_x, &size_y, &size_z);
 
   fprintf_C (out, "#declare TF_X_SCALE = %f;\n", size_x);
-  fprintf_C (out, "#declare TF_Y_SCALE = %f;\n", size_y);
+  // handled by defaults.inc 
+  //fprintf_C (out, "#declare TF_Y_SCALE = %f;\n", size_y); 
   fprintf_C (out, "#declare TF_Z_SCALE = %f;\n", size_z);
 
   fprintf_C (out, "#declare TF_CAMERA_LOCATION = <%f, %f, %f>;\n", 
-             terrain->camera_x * size_x,
-             terrain->camera_y * size_y,
-             terrain->camera_z * size_z);
+             ropt->camera_x * size_x,
+             ropt->camera_y * size_y,
+             ropt->camera_z * size_z);
 
   fprintf_C (out, "#declare TF_CAMERA_LOOK_AT = <%f, %f, %f>;\n", 
-             terrain->lookat_x * size_x,
-             terrain->lookat_y * size_y,
-             terrain->lookat_z * size_z);
+             ropt->lookat_x * size_x,
+             ropt->lookat_y * size_y,
+             ropt->lookat_z * size_z);
 
   fprintf (out, "\n\n");
-  /* fprintf (out, "#include \"%s/%s\"",
-     TERRAFORM_DATA_DIR "/themes", terrain->theme_file); */
-  fprintf (out, "#include \"%s\"", terrain->theme_file);
-  fprintf (out, "\n\n");
 
-  array = terrain->objects;
+  fprintf (out, "#include \"colors.inc\"\n");
+  fprintf (out, "#include \"defaults.inc\"\n");
+
+  fprintf (out, "#include \"%s\"\n", ropt->atmosphere_file);
+  fprintf (out, "#include \"%s\"\n", ropt->sky_file);
+  fprintf (out, "#include \"%s\"\n", ropt->texture_file);
+  fprintf (out, "#include \"%s\"\n", ropt->water_file);
+
+  fprintf (out, "#include \"generic_land.inc\"\n");
+
+  fprintf (out, "\n");
+
+  object_array = terrain->objects;
 
   /* Write out all the #includes */
   includes = g_tree_new ((GCompareFunc) strcmp);
-  for (i = 0; i < array->len; i++)
+  for (i = 0; i < object_array->len; i++)
     {
       TTerrainObject *object;
 
-      object = &g_array_index (array, TTerrainObject, i);
+      object = &g_array_index (object_array, TTerrainObject, i);
       if (g_tree_lookup (includes, object->name) == NULL)
         {
           g_tree_insert (includes, object->name, object->name);
@@ -440,7 +549,8 @@ t_terrain_export_pov (TTerrain *terrain,
 
   fprintf (out, "\n");
 
-  for (i = 0; i < array->len; i++)
+  /* Write out all the placed objects */
+  for (i = 0; i < object_array->len; i++)
     {
       TTerrainObject *object;
       gint            hf_x, hf_y;
@@ -448,12 +558,14 @@ t_terrain_export_pov (TTerrain *terrain,
       gchar          *filename, *name;
       gboolean        macro;
 
-      object = &g_array_index (array, TTerrainObject, i);
+      object = &g_array_index (object_array, TTerrainObject, i);
 
       hf_x = object->x * (terrain->width - 1);
       hf_y = object->y * (terrain->height - 1);
       pov_x = object->x * size_x;
-      pov_y = terrain->heightfield[hf_y * terrain->width + hf_x] * size_y;
+      // write the raw elevation value and let POVRay do the scaling later
+      //pov_y = terrain->heightfield[hf_y * terrain->width + hf_x] * size_y;
+      pov_y = terrain->heightfield[hf_y * terrain->width + hf_x];
       pov_z = (1.0 - object->y) * size_z;
 
       filename = filename_get_base (object->name);
@@ -466,7 +578,7 @@ t_terrain_export_pov (TTerrain *terrain,
         macro = TRUE;
 
       /* FIXME: Handle macros */
-      fprintf_C (out, "object { %s%s() scale <%f, %f, %f> rotate y * %f translate <%f, %f, %f> }\n",
+      fprintf_C (out, "object { %s%s() scale <%f, %f, %f> rotate y * %f translate <%f, TF_Y_SCALE*%f, %f> }\n",
         name, macro ? "()" : "",
         object->scale_x, object->scale_y, object->scale_z,
         object->angle, pov_x, pov_y, pov_z);
@@ -474,12 +586,206 @@ t_terrain_export_pov (TTerrain *terrain,
       g_free (name);
     }
 
+  /* handle placed rivers */
+  if (terrain->riverfield != NULL)
+    {
+      gchar *river, *temp;
+
+      // river heightfield name
+      temp = filename_get_base_without_extension_and_dot (filename);
+      river = g_strdup_printf( "%s_river.tga", temp);
+      g_free (temp);
+
+      // write river heightfield
+      if (t_terrain_export_tga_field (terrain, river, terrain->riverfield) != 0)
+        status = -1;
+
+      if (status != -1)
+        {
+        fprintf (out, 
+           "#declare RIVER_HEIGHT_FIELD = height_field { tga \"%s\" smooth }\n",
+           river);
+        fprintf (out, "#include \"river_texture.inc\"\n");
+        fprintf (out, "#include \"generic_river.inc\"\n");
+	}
+
+      g_free (river);
+    } // end river
+
   status = ferror (out);
   fclose (out);
 
   /* restore the original locale */
   setlocale (LC_NUMERIC, previous_locale); 
 
+  /* Write TGA file */
+  if (t_terrain_export_tga (terrain, tga) != 0)
+    status = -1;
+
+  g_free (tga);
+
+  return status;
+}
+
+/* write a povray ini file to drive a POV rendering */
+static gint
+t_terrain_export_ini (TTerrain  *terrain,
+                      gchar     *filename)
+{
+  FILE  *ini_file;
+  gint   status;
+  gchar *params;
+  gchar *home_dir;
+  gchar *pov;
+  time_t now;
+  TTerrainOptRender * const ropt = &(terrain->render_options);
+
+  /* Generate filenames */
+  if (strlen(ropt->povray_filename) < 1 )
+    ropt->povray_filename = strdup(filename);
+  pov = filename_new_extension (ropt->povray_filename, "pov");
+
+  time(&now);
+  params = g_strdup_printf (";; This ini file generated by Terraform on %s\n\n",
+		             ctime(&now));
+  params = g_strdup_printf ("%sWidth=%d\n", params, ropt->image_width);
+  params = g_strdup_printf ("%sHeight=%d\n", params, ropt->image_height);
+  params = g_strdup_printf ("%sQuality=%d\n", params, ropt->render_quality);
+
+  if (ropt->do_aa)
+    {
+      params = g_strdup_printf ("%sAntialias=on\n", params);
+      params = g_strdup_printf ("%sAntialias_Threshold=%f\n", 
+                                params, ropt->aa_threshold);
+
+      if (ropt->do_jitter)
+	{
+	  params = g_strdup_printf ("%sJitter=on\n", params);
+	  params = g_strdup_printf ("%sJitter_Amount=%f\n", 
+                                    params, ropt->jitter_amount);
+	}
+
+      if (ropt->aa_type == 1) /* 0 = type 1, 1 = type 2 */
+	{
+	  params = g_strdup_printf ("%sSampling_Method=2\n", params);
+	  params = g_strdup_printf ("%sAntialias_Depth=%f\n", 
+                                    params, ropt->aa_depth);
+	}
+    }
+
+  if (ropt->filetype == 0) // PNG
+    {
+      params = g_strdup_printf ("%sOutput_File_Type=N\n", params);
+      params = g_strdup_printf ("%sOutput_File_Name=%s\n", params, 
+				filename_new_extension (pov, "png"));
+      /* I hope this is the best solution for the "out" variable */
+    }
+  else 
+    if (ropt->filetype == 1) //PPM
+    {
+      params = g_strdup_printf ("%sOutput_File_Type=P\n", params);
+      params = g_strdup_printf ("%sOutput_File_Name=%s\n", params, 
+				filename_new_extension (pov, "ppm"));
+    }
+  else 
+    if (ropt->filetype == 2) //TGA
+    {
+      params = g_strdup_printf ("%sOutput_File_Type=T\n", params);
+      params = g_strdup_printf ("%sOutput_File_Name=%s\n", params, 
+				filename_new_extension (pov, "tga"));
+    }
+  else 
+    if (ropt->filetype == 3) //TGA (compressed)
+    {
+      params = g_strdup_printf ("%sOutput_File_Type=C\n", params);
+      params = g_strdup_printf ("%sOutput_File_Name=%s\n", params, 
+				filename_new_extension (pov, "tga"));
+    }
+
+  // params = g_strdup_printf ("%sOutput_File_Name=%s\n", params, out); 
+  params = g_strdup_printf ("%sInput_File_Name=%s\n", params, pov);
+  params = g_strdup_printf ("%sDisplay=on\n", params);
+  params = g_strdup_printf ("%sPause_When_Done=on\n\n", params);
+
+  /* povray files installed under the $HOME dir */
+  home_dir = getenv ("HOME");
+  if (home_dir)
+    {
+      gchar       *terraform_dir;
+      struct stat  statdir;
+      gint         rc;
+
+      terraform_dir = g_strdup_printf ("%s/.terraform", home_dir);
+      rc = stat (terraform_dir, &statdir);
+
+      if (rc != -1 && S_ISDIR (statdir.st_mode))
+        {
+        gchar *td = terraform_dir; /* shorthand notation */
+        gchar *object_dir = g_strdup_printf ("%s/objects", td);
+        gchar *include_dir = g_strdup_printf ("%s/include", td);
+
+        gchar *atmosphere_dir = g_strdup_printf ("%s/include/atmospheres", td);
+        gchar *textures_dir = g_strdup_printf ("%s/include/earth_textures", td);
+        gchar *skies_dir = g_strdup_printf ("%s/include/skies", td);
+        gchar *clouds_dir = g_strdup_printf ("%s/include/skies/include", td);
+        gchar *water_dir = g_strdup_printf ("%s/include/water", td);
+        gchar *img_map_dir = g_strdup_printf ("%s/image_maps", td);
+
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params, include_dir);
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params, object_dir);
+
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params,atmosphere_dir);
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params, textures_dir);
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params, skies_dir);
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params, clouds_dir);
+        params = g_strdup_printf ("%sLibrary_Path=%s\n", params, water_dir);
+        params = g_strdup_printf ("%sLibrary_Path=%s\n\n", params, img_map_dir);
+
+        g_free (include_dir);
+        g_free (object_dir);
+        }
+
+      g_free (terraform_dir);
+    }
+
+  /* global installed povray files */
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/include");
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/objects");
+
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/include/atmospheres");
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/include/earth_textures");
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/include/skies");
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/include/skies/include");
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/include/water");
+  params = g_strdup_printf ("%sLibrary_Path=%s\n", params, 
+                            TERRAFORM_DATA_DIR "/image_maps");
+
+  /* write ini file */
+  ini_file = fopen (filename, "w");
+  status = -1;
+  if (ini_file != NULL)
+    {
+      /* g_print ("%s\n", params); */
+      fprintf (ini_file, "%s\n", params);
+      status = ferror (ini_file);
+      fclose (ini_file);
+    }
+
+  /* write POV file */
+  if (t_terrain_export_pov (terrain, pov) != 0)
+    status = -1;
+
+  g_free (params);
+  //g_free (out);
+  g_free (pov);
+
   return status;
 }
 
@@ -710,6 +1016,48 @@ t_terrain_export_pg8 (TTerrain *terrain,
   return status;
 }
 
+static gint
+t_terrain_export_ppm (TTerrain *terrain,
+                      gchar    *filename)
+{
+  int        x, y;
+  gint32     idx;
+  FILE      *out;
+  int        status;
+  gchar      buf[3];
+
+  /* *** open PPM file *** */
+  out = fopen (filename, "wb");
+  if (!out)
+    return -1;
+
+  fprintf (out, "P6\n"); /* binary PPM image */
+  fprintf (out, "# Image exported by Terraform\n");
+  fprintf (out, "%d %d\n", terrain->width, terrain->height);
+  fprintf (out, "%d\n", MAX_1ST_BYTE);
+
+  idx = 0;
+  buf[0] = 0;
+  for (y = 0; y < terrain->height; y++) 
+    for (x = 0; x < terrain->width; x++) 
+      {
+      gint value = 
+        (gint) MIN (MAX (MAX_16_BIT*terrain->heightfield[idx],0.0), MAX_16_BIT);
+
+      buf[0] = value;
+      buf[1] = value >> 8;
+      buf[2] = value >> 16;
+
+      fwrite (buf, 3, 1, out);
+      ++idx;
+      }
+
+  status = ferror (out);
+  fclose (out);
+
+  return status;
+}
+
 /* defs for MATLAB file format */
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 #  define NUM_FMT 1 /* 1 IEEE Big Endian (SPARC, SGI, Motorola, etc.) */
